"""Store various application values."""
from typing import List


class AppKeys:
    """Store application keys in one place."""

    GITHUB_TOKEN = 'GITHUB_TOKEN'
    VAULT_ADDR = 'VAULT_ADDR'

    IMAGE_API_RELATIVE_URL = 'image_api_relative_url'
    IMAGE_ENHANCE_API_RELATIVE_URL = 'image_enhance_api_relative_url'


class LouvreSecretPathNames:
    """Store the secret path names that need to be set as environment variables 
    in order to use this package."""

    ISSUER = 'ISSUER'

    TOKEN_ENDPOINT = 'TOKEN_ENDPOINT'
    CLIENT_ID_API = 'CLIENT_ID_API'
    CLIENT_SECRET_API = 'CLIENT_SECRET_API'
    LOUVRE_DOMAIN = 'LOUVRE_DOMAIN'

    # Training against PROD
    TOKEN_ENDPOINT_PROD = 'TOKEN_ENDPOINT_PROD'
    CLIENT_ID_API_PROD = 'CLIENT_ID_API_PROD'
    CLIENT_SECRET_API_PROD = 'CLIENT_SECRET_API_PROD'
    LOUVRE_DOMAIN_PROD = 'LOUVRE_DOMAIN_PROD'

    @classmethod
    def get_all(cls) -> List[str]:
        """
        Return all the class members.
        """
        return [
            attr for attr in dir(cls)
            if not callable(getattr(cls, attr)) and not attr.startswith("__")
        ]


class ImageVariants:
    """Allowed values for the image variant parameter."""

    STANDARD = 'standard'
    THUMBNAIL = 'thumbnail'
    ORIGINAL = 'original'
    DEFAULT = STANDARD



"""This class is in its own module to avoid import issues."""

import os
from .enums import LouvreSecretPathNames


class SecretPaths:
    """Store secret paths used against the vault service."""

    TOKEN_ENDPOINT = os.environ[LouvreSecretPathNames.TOKEN_ENDPOINT]
    CLIENT_ID_API = os.environ[LouvreSecretPathNames.CLIENT_ID_API]
    CLIENT_SECRET_API = os.environ[LouvreSecretPathNames.CLIENT_SECRET_API]
    LOUVRE_DOMAIN = os.environ[LouvreSecretPathNames.LOUVRE_DOMAIN]

    # Against PROD
    TOKEN_ENDPOINT_PROD = os.environ[LouvreSecretPathNames.TOKEN_ENDPOINT_PROD]
    CLIENT_ID_API_PROD = os.environ[LouvreSecretPathNames.CLIENT_ID_API_PROD]
    CLIENT_SECRET_API_PROD = os.environ[
        LouvreSecretPathNames.CLIENT_SECRET_API_PROD]
    LOUVRE_DOMAIN_PROD = os.environ[LouvreSecretPathNames.LOUVRE_DOMAIN_PROD]

    ISSUER = os.environ[LouvreSecretPathNames.ISSUER]



"""
Module with methods to authenticate against ImageAPI and ImageEnhanceAPI.
"""

from datetime import datetime, timedelta
from typing import Dict
import requests
from .config import Config
from .secret_paths import SecretPaths


class Token():
    """Class for working with jwt tokens."""

    _tokens: Dict[str, str] = {}
    _expiry: Dict[str, datetime] = {}

    @classmethod
    def get_token(cls, using_production_images: bool = False) -> str:
        """Return the current token if still valid, else a new one.
           Assume that the token life has ended when time is not smaller than:
           _expiry minus the buffer"""

        endpoint: str = SecretPaths.TOKEN_ENDPOINT_PROD if using_production_images else SecretPaths.TOKEN_ENDPOINT
        if endpoint in cls._tokens.keys() and endpoint in cls._expiry.keys(
        ) and datetime.now() < cls._expiry[endpoint] - timedelta(
                seconds=Config.token_buffer_seconds):
            return cls._tokens[endpoint]
        return cls.get_new_token(
            using_production_images=using_production_images)

    @classmethod
    def get_new_token(cls, using_production_images: bool = False) -> str:
        """Create and get token for this API to communicate with ImageAPI and ImageEnhanceAPI.
           Assume the expiry time is found by adding expires_in to the time right before triggering the request."""
        tick = datetime.now()
        token_response = requests.post(
            url=Config.get_secret(
                SecretPaths.TOKEN_ENDPOINT_PROD
                if using_production_images else SecretPaths.TOKEN_ENDPOINT),
            headers={},
            data={
                'grant_type':
                'client_credentials',
                'client_id':
                Config.get_secret(
                    SecretPaths.CLIENT_ID_API_PROD
                    if using_production_images else SecretPaths.CLIENT_ID_API),
                'client_secret':
                Config.get_secret(
                    SecretPaths.CLIENT_SECRET_API_PROD if
                    using_production_images else SecretPaths.CLIENT_SECRET_API)
            })

        endpoint: str = SecretPaths.TOKEN_ENDPOINT_PROD if using_production_images else SecretPaths.TOKEN_ENDPOINT
        cls._tokens[endpoint] = token_response.json()['access_token']
        cls._expiry[endpoint] = tick + timedelta(
            seconds=int(token_response.json()['expires_in']))
        return cls._tokens[endpoint]

    @classmethod
    def get_headers(cls, using_production_images: bool = False) -> dict:
        """Return headers to be used in HTTP requests against ImageAPI / ImageEnhanceAPI."""
        return {
            "Authorization":
            "Bearer {}".format(
                cls.get_token(
                    using_production_images=using_production_images)),
            'Content-Type':
            'application/json'
        }
